@@ -20,7 +20,7 @@
  */
 
 #include "P_OCSPStapling.h"
-#ifdef HAVE_OPENSSL_OCSP_STAPLING
+#ifdef TS_USE_TLS_OCSP
 
 #include <openssl/ssl.h>
 #include <openssl/ocsp.h>
@@ -51,12 +51,15 @@ certinfo_free(void * /*parent*/, void *ptr, CRYPTO_EX_DATA * /*ad*/, int /*idx*/
 {
   certinfo *cinf = (certinfo *)ptr;
 
-  if (!cinf)
+  if (!cinf) {
     return;
-  if (cinf->uri)
+  }
+  if (cinf->uri) {
     OPENSSL_free(cinf->uri);
-  if (cinf->certname)
+  }
+  if (cinf->certname) {
     ats_free(cinf->certname);
+  }
   ink_mutex_destroy(&cinf->stapling_mutex);
   OPENSSL_free(cinf);
 }
@@ -66,8 +69,9 @@ static int ssl_stapling_index = -1;
 void
 ssl_stapling_ex_init()
 {
-  if (ssl_stapling_index != -1)
+  if (ssl_stapling_index != -1) {
     return;
+  }
   ssl_stapling_index = SSL_CTX_get_ex_new_index(0, nullptr, nullptr, nullptr, certinfo_free);
 }
 
@@ -116,9 +120,6 @@ stapling_get_issuer(SSL_CTX *ssl_ctx, X509 *x)
     }
   }
 
-  if (!X509_STORE_CTX_init(inctx, st, nullptr, nullptr)) {
-    goto end;
-  }
   if (X509_STORE_CTX_get1_issuer(&issuer, inctx, x) <= 0) {
     issuer = nullptr;
   }
@@ -158,19 +159,21 @@ ssl_stapling_init_cert(SSL_CTX *ctx, X509 *cert, const char *certname)
   cinf->uri         = nullptr;
   cinf->certname    = ats_strdup(certname);
   cinf->resp_derlen = 0;
-  ink_mutex_init(&cinf->stapling_mutex, "stapling_mutex");
+  ink_mutex_init(&cinf->stapling_mutex);
   cinf->is_expire   = true;
   cinf->expire_time = 0;
 
   issuer = stapling_get_issuer(ctx, cert);
   if (issuer == nullptr) {
     Note("cannot get issuer certificate from %s", certname);
-    return false;
+    goto err;
   }
 
   cinf->cid = OCSP_cert_to_id(nullptr, cert, issuer);
-  if (!cinf->cid)
-    return false;
+  if (!cinf->cid) {
+    goto err;
+  }
+
   X509_digest(cert, EVP_sha1(), cinf->idx, nullptr);
 
   aia = X509_get1_ocsp(cert);
@@ -180,14 +183,28 @@ ssl_stapling_init_cert(SSL_CTX *ctx, X509 *cert, const char *certname)
   }
 
   if (!cinf->uri) {
-    Note("no responder URI for %s", certname);
-    return false;
+    Note("no OCSP responder URI for %s", certname);
+    goto err;
   }
 
   SSL_CTX_set_ex_data(ctx, ssl_stapling_index, cinf);
 
-  Note("successfully initialized certinfo for %s into SSL_CTX: %p", certname, ctx);
+  Note("successfully initialized stapling for %s into SSL_CTX: %p", certname, ctx);
   return true;
+
+err:
+  if (cinf->uri) {
+    OCSP_CERTID_free(cinf->cid);
+  }
+
+  if (cinf->certname) {
+    ats_free(cinf->certname);
+  }
+
+  if (cinf) {
+    OPENSSL_free(cinf);
+  }
+  return false;
 }
 
 static certinfo *
@@ -196,8 +213,9 @@ stapling_get_cert_info(SSL_CTX *ctx)
   certinfo *cinf;
 
   cinf = (certinfo *)SSL_CTX_get_ex_data(ctx, ssl_stapling_index);
-  if (cinf && cinf->cid)
+  if (cinf && cinf->cid) {
     return cinf;
+  }
 
   return nullptr;
 }
@@ -300,8 +318,9 @@ query_responder(BIO *b, char *host, char *path, OCSP_REQUEST *req, int req_timeo
 
   OCSP_REQ_CTX_free(ctx);
 
-  if (rv)
+  if (rv) {
     return resp;
+  }
 
   return nullptr;
 }
@@ -315,8 +334,9 @@ process_responder(OCSP_REQUEST *req, char *host, char *path, char *port, int req
   if (!cbio) {
     goto end;
   }
-  if (port)
+  if (port) {
     BIO_set_conn_port(cbio, port);
+  }
 
   BIO_set_nbio(cbio, 1);
   if (BIO_do_connect(cbio) <= 0 && !BIO_should_retry(cbio)) {
@@ -326,8 +346,9 @@ process_responder(OCSP_REQUEST *req, char *host, char *path, char *port, int req
   resp = query_responder(cbio, host, path, req, req_timeout);
 
 end:
-  if (cbio)
+  if (cbio) {
     BIO_free_all(cbio);
+  }
   return resp;
 }
 
@@ -337,7 +358,7 @@ stapling_refresh_response(certinfo *cinf, OCSP_RESPONSE **prsp)
   bool rv           = true;
   OCSP_REQUEST *req = nullptr;
   OCSP_CERTID *id   = nullptr;
-  char *host, *path, *port;
+  char *host = nullptr, *port = nullptr, *path = nullptr;
   int ssl_flag    = 0;
   int req_timeout = -1;
 
@@ -349,13 +370,16 @@ stapling_refresh_response(certinfo *cinf, OCSP_RESPONSE **prsp)
   }
 
   req = OCSP_REQUEST_new();
-  if (!req)
+  if (!req) {
     goto err;
+  }
   id = OCSP_CERTID_dup(cinf->cid);
-  if (!id)
+  if (!id) {
     goto err;
-  if (!OCSP_request_add0_id(req, id))
+  }
+  if (!OCSP_request_add0_id(req, id)) {
     goto err;
+  }
 
   req_timeout = SSLConfigParams::ssl_ocsp_request_timeout;
   *prsp       = process_responder(req, host, path, port, req_timeout);
@@ -377,18 +401,23 @@ stapling_refresh_response(certinfo *cinf, OCSP_RESPONSE **prsp)
   } else {
     Debug("ssl_ocsp", "stapling_refresh_response: successful refresh OCSP response");
   }
-
-done:
-  if (req)
-    OCSP_REQUEST_free(req);
-  if (*prsp)
-    OCSP_RESPONSE_free(*prsp);
-  return rv;
+  goto done;
 
 err:
   rv = false;
   Error("stapling_refresh_response: failed to refresh OCSP response");
-  goto done;
+
+done:
+  if (req) {
+    OCSP_REQUEST_free(req);
+  }
+  if (*prsp) {
+    OCSP_RESPONSE_free(*prsp);
+  }
+  OPENSSL_free(host);
+  OPENSSL_free(path);
+  OPENSSL_free(port);
+  return rv;
 }
 
 void
@@ -460,4 +489,4 @@ ssl_callback_ocsp_stapling(SSL *ssl)
   }
 }
 
-#endif /* HAVE_OPENSSL_OCSP_STAPLING */
+#endif /* TS_USE_TLS_OCSP */
