@@ -25,8 +25,8 @@ limitations under the License.
 #include <sys/stat.h>
 
 #include "ts/ts.h"
-#include "ts/ink_platform.h"
-#include "ts/ink_defs.h"
+#include "tscore/ink_platform.h"
+#include "tscore/ink_defs.h"
 
 static const char PLUGIN_NAME[]      = "acme";
 static const char ACME_WK_PATH[]     = ".well-known/acme-challenge/";
@@ -84,7 +84,7 @@ make_absolute_path(char *dest, int dest_len, const char *file, int file_len)
     }
   }
 
-  return snprintf(dest, dest_len - 1, "%s/%.*s", gConfig.proof, file_len, file);
+  return snprintf(dest, dest_len, "%s/%.*s", gConfig.proof, file_len, file);
 }
 
 static void
@@ -202,7 +202,7 @@ acme_process_write(TSCont contp, TSEvent event, AcmeState *my_state)
     char buf[64]; /* Plenty of space for CL: header */
     int len;
 
-    len = snprintf(buf, sizeof(buf) - 1, "Content-Length: %zd\r\n\r\n", (size_t)my_state->stat_buf.st_size);
+    len = snprintf(buf, sizeof(buf), "Content-Length: %zd\r\n\r\n", (size_t)my_state->stat_buf.st_size);
     my_state->output_bytes += add_data_to_resp(buf, len, my_state);
     my_state->output_bytes += add_file_to_resp(my_state);
 
@@ -303,7 +303,8 @@ TSPluginInit(int argc, const char *argv[])
   const char *proof = "acme";
 
   static const struct option longopt[] = {
-    {(char *)"proof-directory", optional_argument, NULL, 'p'}, {NULL, no_argument, NULL, '\0'},
+    {(char *)"proof-directory", optional_argument, NULL, 'p'},
+    {NULL, no_argument, NULL, '\0'},
   };
 
   memset(&gConfig, 0, sizeof(gConfig));
@@ -326,7 +327,7 @@ TSPluginInit(int argc, const char *argv[])
     int len             = strlen(proof) + strlen(confdir) + 8;
 
     gConfig.proof = TSmalloc(len);
-    snprintf(gConfig.proof, len - 1, "%s/%s", confdir, proof);
+    snprintf(gConfig.proof, len, "%s/%s", confdir, proof);
     TSDebug(PLUGIN_NAME, "base directory for proof-types is %s", gConfig.proof);
   } else {
     gConfig.proof = TSstrdup(proof);
@@ -337,7 +338,7 @@ TSPluginInit(int argc, const char *argv[])
   info.support_email = "dev@trafficserver.apache.org";
 
   if (TS_SUCCESS != TSPluginRegister(&info)) {
-    TSError("[%s] Plugin registration failed.", PLUGIN_NAME);
+    TSError("[%s] Plugin registration failed", PLUGIN_NAME);
     return;
   }
 
