@@ -23,8 +23,9 @@
 
 #pragma once
 
-#include "ts/ink_platform.h"
-#include "ts/ink_resolver.h"
+#include "tscore/ink_platform.h"
+#include "tscore/ink_resolver.h"
+#include <string_view>
 #include "P_Net.h"
 #include "InkAPIInternal.h"
 #include "http/HttpServerSession.h"
@@ -32,11 +33,42 @@
 // Emit a debug message conditional on whether this particular client session
 // has debugging enabled. This should only be called from within a client session
 // member function.
-#define DebugSsn(ssn, tag, ...) DebugSpecific((ssn)->debug(), tag, __VA_ARGS__)
+#define SsnDebug(ssn, tag, ...) SpecificDebug((ssn)->debug(), tag, __VA_ARGS__)
 
 class ProxyClientTransaction;
 struct AclRecord;
 
+enum class ProxyErrorClass {
+  NONE,
+  SSN,
+  TXN,
+};
+
+struct ProxyError {
+  ProxyError() {}
+  ProxyError(ProxyErrorClass cl, uint32_t co) : cls(cl), code(co) {}
+  size_t
+  str(char *buf, size_t buf_len) const
+  {
+    size_t len = 0;
+
+    if (this->cls == ProxyErrorClass::NONE) {
+      buf[0] = '-';
+      return 1;
+    }
+
+    buf[0] = (this->cls == ProxyErrorClass::SSN) ? 'S' : 'T';
+    ++len;
+
+    len += snprintf(buf + len, buf_len - len, "%" PRIx32, this->code);
+
+    return len;
+  }
+
+  ProxyErrorClass cls = ProxyErrorClass::NONE;
+  uint32_t code       = 0;
+};
+
 class ProxyClientSession : public VConnection
 {
 public:
@@ -87,6 +119,12 @@ public:
     user_args[ix] = arg;
   }
 
+  void
+  set_debug(bool flag)
+  {
+    debug_on = flag;
+  }
+
   // Return whether debugging is enabled for this session.
   bool
   debug() const
@@ -112,6 +150,16 @@ public:
     return m_active;
   }
 
+  bool
+  is_draining() const
+  {
+    RecInt draining;
+    if (RecGetRecordInt("proxy.node.config.draining", &draining) != REC_ERR_OKAY) {
+      return false;
+    }
+    return draining != 0;
+  }
+
   // Initiate an API hook invocation.
   void do_api_callout(TSHttpHookID id);
 
@@ -122,6 +170,12 @@ public:
     return false;
   }
 
+  virtual bool
+  is_chunked_encoding_supported() const
+  {
+    return false;
+  }
+
   // Override if your session protocol cares.
   virtual void
   set_half_close_flag(bool flag)
@@ -137,6 +191,24 @@ public:
   // Indicate we are done with a transaction.
   virtual void release(ProxyClientTransaction *trans) = 0;
 
+  virtual in_port_t
+  get_outbound_port() const
+  {
+    return outbound_port;
+  }
+
+  virtual IpAddr
+  get_outbound_ip4() const
+  {
+    return outbound_ip4;
+  }
+
+  virtual IpAddr
+  get_outbound_ip6() const
+  {
+    return outbound_ip6;
+  }
+
   int64_t
   connection_id() const
   {
@@ -151,7 +223,7 @@ public:
   virtual HttpServerSession *
   get_server_session() const
   {
-    return NULL;
+    return nullptr;
   }
 
   TSHttpHookID
@@ -178,18 +250,18 @@ public:
   bool
   is_client_closed() const
   {
-    return get_netvc() == NULL;
+    return get_netvc() == nullptr;
   }
 
   virtual int
-  populate_protocol(ts::StringView *result, int size) const
+  populate_protocol(std::string_view *result, int size) const
   {
     auto vc = this->get_netvc();
     return vc ? vc->populate_protocol(result, size) : 0;
   }
 
   virtual const char *
-  protocol_contains(ts::StringView tag_prefix) const
+  protocol_contains(std::string_view tag_prefix) const
   {
     auto vc = this->get_netvc();
     return vc ? vc->protocol_contains(tag_prefix) : nullptr;
@@ -200,45 +272,64 @@ public:
 
   static int64_t next_connection_id();
 
+  virtual sockaddr const *
+  get_client_addr()
+  {
+    NetVConnection *netvc = get_netvc();
+    return netvc ? netvc->get_remote_addr() : nullptr;
+  }
+  virtual sockaddr const *
+  get_local_addr()
+  {
+    NetVConnection *netvc = get_netvc();
+    return netvc ? netvc->get_local_addr() : nullptr;
+  }
+
   /// acl record - cache IpAllow::match() call
-  const AclRecord *acl_record;
+  const AclRecord *acl_record = nullptr;
+
+  /// Local address for outbound connection.
+  IpAddr outbound_ip4;
+  /// Local address for outbound connection.
+  IpAddr outbound_ip6;
+  /// Local port for outbound connection.
+  in_port_t outbound_port{0};
 
   /// DNS resolution preferences.
-  HostResStyle host_res_style;
+  HostResStyle host_res_style = HOST_RES_IPV4;
 
-  ink_hrtime ssn_start_time;
-  ink_hrtime ssn_last_txn_time;
+  ink_hrtime ssn_start_time    = 0;
+  ink_hrtime ssn_last_txn_time = 0;
+
+  // noncopyable
+  ProxyClientSession(ProxyClientSession &) = delete;
+  ProxyClientSession &operator=(const ProxyClientSession &) = delete;
 
 protected:
   // XXX Consider using a bitwise flags variable for the following flags, so
   // that we can make the best use of internal alignment padding.
 
   // Session specific debug flag.
-  bool debug_on;
-  bool hooks_on;
-  bool in_destroy;
+  bool debug_on   = false;
+  bool hooks_on   = true;
+  bool in_destroy = false;
 
-  int64_t con_id;
-  Event *schedule_event;
+  int64_t con_id        = 0;
+  Event *schedule_event = nullptr;
 
 private:
-  ProxyClientSession(ProxyClientSession &);                  // noncopyable
-  ProxyClientSession &operator=(const ProxyClientSession &); // noncopyable
-
   void handle_api_return(int event);
   int state_api_callout(int event, void *edata);
 
-  APIHookScope api_scope;
-  TSHttpHookID api_hookid;
-  APIHook *api_current;
+  APIHookScope api_scope  = API_HOOK_SCOPE_NONE;
+  TSHttpHookID api_hookid = TS_HTTP_READ_REQUEST_HDR_HOOK;
+  APIHook *api_current    = nullptr;
   HttpAPIHooks api_hooks;
-  void *user_args[HTTP_SSN_TXN_MAX_USER_ARG];
+  void *user_args[TS_HTTP_MAX_USER_ARG];
 
   // for DI. An active connection is one that a request has
   // been successfully parsed (PARSE_DONE) and it remains to
   // be active until the transaction goes through or the client
   // aborts.
-  bool m_active;
-
-  friend void TSHttpSsnDebugSet(TSHttpSsn, int);
+  bool m_active = false;
 };
